

#define  F_CPU 8000000UL
#include <util/delay.h>
#include <stdint.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "Z_USART.h"
#include "Z_SPI.h"
#include "w5500.h"
#include "socket.h"
#include "webpage.h"


#define HTTPD_BUF_SIZE	128
#define MAX_URI_SIZE 15


/*********************************************
* HTTP Method 
*********************************************/
#define		METHOD_ERR		0		/**< Error Method. */
#define		METHOD_GET		1		/**< GET Method.   */
#define		METHOD_HEAD		2		/**< HEAD Method.  */
#define		METHOD_POST		3		/**< POST Method.  */


static uint8_t http_request[512];

uint8_t	http_ParsedRequest_METHOD;
char http_ParsedRequest_URI[MAX_URI_SIZE];

void httpServer_run(uint8_t seqnum);
void parse_http_request(uint8_t * request);
static void http_handler(uint8_t s, char* p_http_request);
void html_send(char* page);
int32_t NetSend(uint8_t sn, uint8_t * buf, uint16_t len);



void httpServer_run(uint8_t seqnum)
{
	uint8_t s;	// socket number
	uint16_t len;

	// Get the H/W socket number
	s = 0;  //Use Socket 0.  The web server socket

	/* HTTP Service Start */
	switch(getSn_SR(s))
	{
		case SOCK_ESTABLISHED:
		UART_SendString("Socket established\n\r");
			// Interrupt clear
			if(getSn_IR(s) & Sn_IR_CON)	{setSn_IR(s, Sn_IR_CON);}		
			// HTTP Process states			
			if ((len = getSn_RX_RSR(s)) > 0)
				{
				if (len > HTTPD_BUF_SIZE) len = HTTPD_BUF_SIZE;
				len = recv(s, http_request, len);
				
				*((http_request) + len) = '\0';  // insert null-terminate symbol to correct parse data
				
				parse_http_request(http_request);
				http_handler(s, http_ParsedRequest_URI);
			
				// Check the TX socket buffer for End of HTTP response sends
				while(getSn_TX_FSR(s) != (getSn_TXBUF_SIZE(s)*1024));
				}
	
			disconnect(s);

		break;

		case SOCK_CLOSE_WAIT:
			disconnect(s);
		break;

		case SOCK_CLOSED:
			if(socket(s, Sn_MR_TCP, 80, 0x00) == s)    /* Recreate the socket the socket */
				{ /* SOCKET recreate successfully */ }
		break;

		case SOCK_INIT:
			listen(s);
		break;

		case SOCK_LISTEN:
		break;

		default :
		break;

	} // end of switch

}


void parse_http_request(uint8_t * request)
{
	char * nexttok;
	nexttok = strtok((char*)request," ");
	
	if(!nexttok) { http_ParsedRequest_METHOD = METHOD_ERR;  return;	}
		
	if      (!strcasecmp(nexttok, "GET"))  { http_ParsedRequest_METHOD = METHOD_GET; nexttok = strtok(NULL," "); }
	else if (!strcasecmp(nexttok, "HEAD")) { http_ParsedRequest_METHOD = METHOD_HEAD;	nexttok = strtok(NULL," ");	}
	else if (!strcasecmp(nexttok, "POST")) { nexttok = strtok(NULL,"\0"); http_ParsedRequest_METHOD = METHOD_POST;	}
	else { http_ParsedRequest_METHOD = METHOD_ERR;	}
	
	strcpy((char *)http_ParsedRequest_URI, nexttok);
}


static void http_handler(uint8_t s, char* p_http_request)
{
	//method Analysis
	switch (http_ParsedRequest_METHOD)
	{
		case METHOD_HEAD :
		case METHOD_GET :	
			html_send(p_http_request);		
		break;

		case METHOD_POST :	
		break;

		default :
		break;
	}
}

void html_send(char* requested_uri)
{
	if (!strcmp(requested_uri, "/")) NetSend(0, (uint8_t *)HOME_PAGE, strlen((char*)HOME_PAGE));  
	else UART_SendString("Page Not Found \n\r");
}

	
int32_t NetSend(uint8_t sn, uint8_t * buf, uint16_t len)
{
	uint16_t freesize=0;
	freesize = getSn_TxMAX(sn);
	while(1)
		{
		freesize = getSn_TX_FSR(sn);
		if(len <= freesize) break;
		}
	
	uint16_t ptr = 0;
	uint32_t addrsel = 0;

	//if(len == 0)  return;
	ptr = getSn_TX_WR(sn);
	addrsel = ((uint32_t)ptr << 8) + (WIZCHIP_TXBUF_BLOCK(sn) << 3);
	
	uint16_t i;

	//WIZCHIP_CRITICAL_ENTER();
	W5500_Select();

	addrsel |= (_W5500_SPI_WRITE_ | _W5500_SPI_VDM_OP_); //Combining Address & Control
	
	//Sending Address & Control
	SPI_SndByte((addrsel & 0x00FF0000) >> 16);  //Address High Byte
	SPI_SndByte((addrsel & 0x0000FF00) >>  8);  //Address low Byte
	SPI_SndByte((addrsel & 0x000000FF) >>  0);	//Control Phase
	_delay_ms(10);
	for(i = 0; i < len; i++)    SPI_SndByte(buf[i]);
		
	W5500_DeSelect();
	//WIZCHIP_CRITICAL_EXIT();
	
	ptr += len;
	setSn_TX_WR(sn,ptr);

	setSn_CR(sn,Sn_CR_SEND);  //Sending the data in the buffer
	
	/* wait to process the command... */
	while(getSn_CR(sn));
	
	return (int32_t)len;
}

